#!/usr/bin/perl

#
# DaveMP web-based music player interface - main web server
#
# Version 1.20
#
# Written by Jay Kuri Copyright 2007
#
# This work is licensed under the Creative Commons
# Attribution-Noncommercial-Share Alike 2.5 License. To view a copy of this
# license, visit http://creativecommons.org/licenses/by-nc-sa/2.5/ or send a
# letter to Creative Commons, 543 Howard Street, 5th Floor, San Francisco,
# California, 94105, USA.


use lib './lib';
use Davemp;
use Davemp::Backend::Mpd;
use Data::Dumper;

sub loadconfig {
    my $filename = shift;
    open FH, $filename || die "Can't open $filename.";
    my @lines = <FH>;
    close FH;
    
    my %configinfo;
    foreach my $line (@lines) {
        chomp $line;
        $line =~ s/[^\\]#//; 
        my ($key, $val) = split /\s*=\s*/, $line;
        $val =~ s/\s*([^\s]*)\s*/$1/;
        $val =~ s/^"(.*)"$/$1/;
        $configinfo{lc($key)} = $val; 
    } 
    #print Dumper(\%configinfo);
    return \%configinfo;
}

## check for existance of config file.

if ( $#ARGV == -1 || ! -e $ARGV[0] ) {
    die "Config file not provided or doesn't exist... \nUsage: $0 path/to/davemp.conf\n";
}

my $config = loadconfig($ARGV[0]);
my $backend = Davemp::Backend::Mpd->new($config);
my $server = Davemp->new();

if (exists($config->{'davemp_http_user'}) && exists($config->{'davemp_http_password'})) {
    print "Username / Password to access player\n"
}

$server->config($config);
$server->setBackend($backend);
if (exists($config->{'davemp_host'}) && $config->{'davemp_host'} ne '') {
    $server->host($config->{'davemp_host'});
}
if (exists($config->{'davemp_port'}) && $config->{'davemp_port'} > 0 ) {
    $server->port($config->{'davemp_port'});
} else {
    $server->port('4691');
}
print "Server started on port " . $server->port . "\n";
$server->background;

exit 0;
