/*
 * The DaveMP web-based music player interface
 *
 * Version 1.20
 *
 * Written by Jay Kuri Copyright 2007
 *
 * This work is licensed under the Creative Commons
 * Attribution-Noncommercial-Share Alike 2.5 License. To view a copy of this
 * license, visit http://creativecommons.org/licenses/by-nc-sa/2.5/ or send a
 * letter to Creative Commons, 543 Howard Street, 5th Floor, San Francisco,
 * California, 94105, USA.
 */   

/* 
    things we need:  
    statusDelegate   - object to poke in order to update status; should respond to obj.setStatus(statusObject)
    
    playlistDelegate - object to poke on playlist operations.  
                       Must respond to: 
                                        setTrackInfo(i, trackinfo);
                                        insertTrack(i, trackinfo);
                                        moveTrack(oldposition, newposition);
                                        deleteTrack(i); 
                                        setActive(position)  - called to indicate which track is playing
                                        setPlaylistLength(length) - called to indicate how long playlist is
                                        
    libraryTreeRoot - Root element of library tree
    libraryTreeNodeclass - class for each node.
    
 */


var DaveMP = new Class({

initialize: function(options) {
    this.options = new Object;
    this.statusDelegate = null;
    this.playlistDelegate = null;
    this.activeLibrary = 'active';
    this.activePlaylist = 'active';
    this.activeOutput = 'active';
    this.libraryTree = null;
    this.playlist = new Array;
    this.currentState = null;
    this.commandQueue = new Array;
    if (options != null) {
        for (var option in options) {
            this.options[option] = options[option];
        }
    }    
},

setStatusDelegate: function(object) {
    this.statusDelegate = object;
},

setPlaylistDelegate: function(object) {
    this.playlistDelegate = object;
},

setupLibrary: function(element, options) {
    if (options == null ) { options = new Object; }
    options.rootElement = $(element);
    this.libraryTree = new LibraryTree(this, options); 
    this.libraryTree.loadRoot();
},

sendCommand: function (command, args, callback) {
    var commandobj = {
        'command':      command,
        'args':         args,
        'callback':     callback
    };
    
    this.commandQueue.push(commandobj);
    if (this.commandQueue.length == 1) {
        this.processCommandQueue();
    }
},

processCommandQueue: function () {
    
    if (this.commandQueue.length > 0) {
        var commandobj = this.commandQueue[0];
        
        var url = "/control/" + commandobj.command;
        if (this.options.urlprefix) {
            url = this.options.urlprefix + url;
        }

        if (commandobj.args) {
            url += "?";
            for (var arg in commandobj.args) {
                url += arg + "=" + encodeURIComponent(commandobj.args[arg]) + "&";
            }
        }
        var callback = commandobj.callback;
        var davemp = this;

        var func = function(request) {
//            var nextitem = function() {
//                this.commandQueue.shift();
//                this.processCommandQueue();
//            };
//            nextitem.bind(this)();
            this.commandQueue.shift();
            if (this.commandQueue.length > 0) {
                this.processCommandQueue();
            }
            var state = Json.evaluate(request);
            callback(state);
        };
        new Ajax(url, { method:"get", onComplete: func.bind(this) }).request();
    }
},

getStatus: function () {
    var statusfunc = function (data) {
        //alert("got status data back. whee!");
        if (data.result) {
            this.currentState = data.result;
            if (this.statusDelegate != null && typeof(this.statusDelegate.setStatus) == 'function') {
                this.statusDelegate.setStatus(data.result);
            }

            if (data.result['playlist'] != null && data.result.playlist['currentposition'] != null && 
                this.playlistDelegate && typeof(this.playlistDelegate.setActive) == 'function') {
                
                this.playlistDelegate.setActive(data.result['playlist']['currentposition']);
            }
        }
        if (this.options['statustimeout'] != null) {
            setTimeout(this.options['statustimeout'], this.getStatus.bind(this));
        }
    }
    if (this.statusDelegate != null) {
        this.sendCommand("status", {}, statusfunc.bind(this));
    }
},
// Track control routines

simpleOutputCommand: function(command, args) {
    var responsefunc = function (data) {
        this.getStatus();
    };
    this.sendCommand("output/" + this.activeOutput + "/" + command, args, responsefunc.bind(this));
},

play: function(track, seek) {
    var args = null;
    if (track != null || seek != null) {
        args = new Object;
        if (track != null) {
            args['track'] = track;
        }
        if (seek != null) {
            args['seek'] = seek;
        }
    }
    this.simpleOutputCommand('play', args);
},

seek: function(seek) {
    var newseek = 0;
    var newtrack = null;
    seek = "" + seek;
    if (seek.substring(0,1) == '+') {
        newseek = this.currentState.output.trackelapsed + seek.substring(1);
        if (newseek >= this.currentState.output.tracklength) {
            newseek = 0;
            newtrack = this.currentState.playlist.currentposition + 1;
            if (newtrack > this.currentState.playlist.length ) {
                // we seeked past the end of the playlist.  we stop.
                this.simpleOutputCommand('stop', null);
                return;
            }
        }
    } else if (seek.substring(0,1) == '-') {
        newseek = this.currentState.output.trackelapsed - seek.substring(1);
    } else {
        newseek = seek;
    }
    if (newseek < 0) {
        newseek = 0;
    }
    this.play(newtrack, newseek);
},

pause: function() {
    this.simpleOutputCommand('pause', null);
},

stop: function() {
    this.simpleOutputCommand('stop', null);
},

next: function() {
    this.simpleOutputCommand('next', null);
},

previous: function() {
    this.simpleOutputCommand('previous', null);
},

setGain: function(level) {
    if (level > 100) { level = 100; }
    if (level < 0) { level = 0; }
    var args = { 'level': level };
    this.simpleOutputCommand('setgain', args);
},

setCrossfade: function(seconds) {
    if (level > 30) { level = 30; }
    if (seconds < 0) { seconds = 0; }
    var args = { 'time': seconds };
    this.simpleOutputCommand('crossfade', args);
},

setRandom: function(state) {
    var args = { 'state': 0 };
    if (state) { 
        args['state'] = 1;
    }
    this.simpleOutputCommand('random', args);
},

setRepeat: function(state) {
    var args = { 'state': 0 };
    if (state) { 
        args['state'] = 1;
    }
    this.simpleOutputCommand('repeat', args);
},

// Playlist related routines 

getPlaylist: function () {
    var playlistfunc = function (data) {
        //alert("got playlist back. whee!");
        if (data.result && this.playlistDelegate) {
            for (var i = 0; i < data.result.length; i++) {
                this.playlistDelegate.setTrackInfo(i, data.result[i]);
            }
        }
        this.playlistDelegate.setPlaylistLength(data.result.length);
    }
    if (this.playlistDelegate) {
        this.sendCommand("playlist/" + this.activePlaylist + "/get", null, playlistfunc.bind(this));
    }
},

addTrackToPlaylist: function (trackid, position) {
    var trackpos = position;
    var func = function (data) {
        if (data.result && this.playlistDelegate && this.playlistDelegate.options.batchupdates != true) {
            this.playlistDelegate.insertTrack(trackpos, data.result);
        }
    };
    var args = {
                  'trackid': trackid,
                  'position': position
    };
    this.sendCommand("playlist/" + this.activePlaylist + "/addtrack", args, func.bind(this));
},

removeTrackFromPlaylist: function (position) {
    var func = function (data) {
        //alert("got info on delete track");
        if (data.result && this.playlistDelegate && this.playlistDelegate.options.batchupdates != true) {
            this.playlistDelegate.deleteTrack(position);
        }
    };
    var args = {
        'position': position
    };
    this.sendCommand("playlist/" + this.activePlaylist + '/removetrack', args, func.bind(this));
},

// moves track at oldposition before the track currently at newposition
// not technically correct, as the track doesn't always wind up at 'newposition' in some cases
// but it is what most people expect when they rearrange things.
moveTrack: function(oldposition, newposition) {

    var func = function (data) {
        //alert("got info on delete track");
        if (data.result && this.playlistDelegate) {
            this.playlistDelegate.moveTrack(oldposition, newposition);
        }
    };

    var args = {
        'oldposition': oldposition,
        'newposition': newposition
    };
    this.sendCommand("playlist/" + this.activePlaylist + "/movetrack", args, func.bind(this));
}
});

// ********* Library Tree code 
var LibraryTree = new Class({
   
    initialize: function(davemp, options) {
        this.davempHandle = davemp;
        this.options = new Object;
        if (options != null) {
            for (var option in options) {
                this.options[option] = options[option];
            }
        }
        if (this.options.nodeClass == null) { 
            this.options.nodeClass = 'library_tree_node';
        }
        this.metadata = this.options.metadata || [ 'artist', 'album', 'title' ];
        this.libraryTreeRoot = new TreeNode($(this.options.rootElement), 
                                            { 
                                                nodeclass: this.options.nodeclass, 
                                                onexpand: this.expandLeaf.bind(this),
                                                onselect: this.addDragNode.bind(this),
                                                ondeselect: this.removeDragNode.bind(this)
                                            });
        var rootdata = this.libraryTreeRoot.setData(new Object);
        rootdata.level = 0;
        this.selectedNodes = new Array;
        rootdata.type = 'root';
        rootdata.metadata = '';           
        rootdata.childsearch = '';
        rootdata.childmeta = this.metadata[0];
        this.libdragger = new JKDrag({
            ondrop: function(dragger, event, library) {
                        if (dragger.currentlyover != null) {
                            library.sendSelectedTo(library.davempHandle.playlistDelegate); //debug("dragged over the playlist");
                        }
                    },
            ghostclass: 'ghost',
            ghostparentelement: $$('body'),
            draggingclass: 'dragging',
            hoverclass: 'draghover',
            hideoriginal: false
        });
        this.libdragger.addDragover(this.davempHandle.playlistDelegate.enclosingelement.parentNode);

        if (this.options.hiderootlibrarylabel) {
            this.libraryTreeRoot.hideLabel();            
            this.libraryTreeRoot.expand();
        }
       
    },
    
    addDragNode: function(node) {
        var nodeelement = node.getNodeElement();
        node.getData().mousedownhandler = function(event) { this.libdragger.startDrag(nodeelement, event, this); };
        node.getData().mousedownhandler = node.getData().mousedownhandler.bind(this);
        nodeelement.addEvent('mousedown', node.getData().mousedownhandler);
    },
    
    removeDragNode: function(node) {
        var nodeelement = node.getNodeElement();
        nodeelement.removeEvent('mousedown', node.getData().mousedownhandler);
        node.getData().mousedownhandler = null;
    },
    
    getTreeRoot: function() {
        return this.libraryTreeRoot;
    },
    
    expandLeaf: function(node) {
        //alert('expanding leaf: ' + node.value);
        if (node.getData().loaded != true) {
            this.loadLeaf(node);
            //node.deleteOption('onexpand');
            node.expand();   
        }
        return true;
    },
    
    createLeaf: function(value, options) {

        // first - let's create the dom elements
        var el = new Element('div');
        var data = new Object;
        data.value = value;
        data.labelelement = new Element('span');
        data.labelelement.style.position = 'relative';
        data.labelelement.disableSelection();
        el.appendChild(data.labelelement);
        data.labelelement.appendText(data.value);
        var node = new TreeNode(el, options);
        node.setData(data);
        return node;
    },
    
    loadLeaf: function(node, loadsubnodes) {

        if (loadsubnodes == null) { loadsubnodes = false }
        
        if (node.getData().loaded != true && node.getData().type != this.metadata[(this.metadata.length-1)]) {
            //var mynode = node;
            node.getData().loaded = true;
            var library = this;
            var obj = new Object;
            obj.library = this;
            obj.node = node;
            obj.loadsubnodes = loadsubnodes;
            var fillnode = function(data) {
                var node = this.node;
                var library = this.library;
                if (data.result) {
                    for (var i=0; i < data.result.length ; i++) {
                        var parentnodedata = node.getData();
                        // create leaf node and set it's name.
                        var subnode = library.createLeaf(data.result[i][parentnodedata.childmeta], node.getAllOptions());
                        // add our subnode to the parent node
                        node.addChild(subnode);
                    
                        var subnodedata = subnode.getData();
                        subnodedata.type = parentnodedata.childmeta;
                        subnodedata.metadata = data.result[i];
                    
                        // this leaves a | at the beginning of all searches.  This could be bad and is probably
                        // the source of the bug that you are trying to track down, as any backend aside from the mpd
                        // backend does not handle leading |'s properly.                
                        subnodedata.childsearch = parentnodedata.childsearch + "|" + subnodedata.type + ":=" + subnodedata.value;
                        subnodedata.level = parentnodedata.level+1;
                        subnodedata.childmeta = library.metadata[subnodedata.level];
                        if (subnodedata.level == (library.metadata.length )) {
                            subnodedata.childmeta = subnodedata.childmeta + ",id";
                        } else {
                            subnode.setExpandable(true,true);
                        }
                        var func = function(event, node) {
                                                        this.selectHandler(event, node);
                                                        //event.stop();
                                                    };
                        subnodedata.labelelement.addEvent('click', func.bindAsEventListener(library, subnode));
                        
                        if (this.loadsubnodes == true) {
                            library.loadLeaf(subnode, true);
                        }
                    }
                }
                //node.deleteOption('onexpand');
                //node.expand();
                return true;
            }

            var args = new Object;
            var childmeta = node.getData().childmeta;
            if (node.getData().level == (library.metadata.length - 1)) {
                childmeta += ',id';
            }
            args.fields = encodeURIComponent(childmeta);
            args.search = encodeURIComponent(node.getData().childsearch);
            //alert(args.length);
            this.davempHandle.sendCommand("library/active/list", args, fillnode.bind(obj))
        } else {
            if (loadsubnodes == true) {
                var childnodes = node.getChildren();
                for (var i = 0; i < childnodes.length; i++) {
                    this.loadLeaf(childnodes[i], true);
                    
                }
            }
        }
    },

    selectHandler: function(event, node) {
        if (event.metaKey == true) {
            //alert('footyfoo');
            if (node.isSelected()) {
    		    node.deselect(true);
    		    var i = this.selectedNodes.indexOf(node);
    		    this.selectedNodes.splice(node, 1);
    		} else {
    			node.select(true);
    			this.selectedNodes.push(node);						    
    		}
        } else {
            if (!node.isSelected()) {
                this.libraryTreeRoot.deselect(true);
                this.selectedNodes.splice(0,this.selectedNodes.length);
                node.select(true);
                this.selectedNodes.push(node);
            }
        }
        if (node.isSelected()) { // && node.getData().type != this.metadata[(this.metadata.length-1)]) {
            this.loadLeaf(node, true);
        }
    },
    
    
    loadRoot: function() {
        //this.libraryTreeRoot = this.createLeaf('Library Root', this.options);
        
        this.loadLeaf(this.libraryTreeRoot);
        this.libraryTreeRoot.setExpandable(true, true);
        
        this.libraryTreeRoot.expand();

        $(this.options.rootElement).replaceWith(this.libraryTreeRoot.getNodeElement());
    },
    
    sendSelectedTo: function(delegate) {
        
        if (delegate && typeof(delegate.acceptItems) == 'function') {
            var selecteditems = new Array;
            var selectedchildren = new Array;
            for (var i=0; i < this.selectedNodes.length; i++) {
                selectedchildren.push(this.selectedNodes[i]);
                selectedchildren.extend(this.selectedNodes[i].getSelectedChildren());
            } 
            
            for (var i=0; i < selectedchildren.length; i++) {
                if (selectedchildren[i].getData().type == this.metadata[(this.metadata.length-1)]) {
                    var metadata = new Object;
                    var existingmeta = selectedchildren[i].getData().metadata;
                    for (var field in existingmeta) {
                        metadata[field] = existingmeta[field];
                    }
                    metadata['library'] = this.davempHandle.activeLibrary;
                    selecteditems.push(metadata);
                }
            }
            return delegate.acceptItems(selecteditems);
        } else {
            return false;
        }  
    }
});

// a little helper thingie - prevents our playlist items from being text-selected when clicking on them.

Element.extend({ 
    
    disableSelection: function () { 
        if (window.ActiveXObject) this.onselectstart = function() {return false}; 
        this.style.MozUserSelect = "none"; return this;
    } 
}); 


var Playlist = new Class({

    initialize: function(davemp, enclosingelement, options) {
        this.davemphandle = davemp;
        this.items = new Array;
        //this.itemelements = new Array;
        //this.selected= new Array;
        this.options = new Object;
        if (options != null) {
            for (var option in options) {
                this.options[option] = options[option];
            }
        }
        this.metadata = this.options.metadata || [ 'artist', 'title', 'album' ];
        this.enclosingelement = enclosingelement;
        this.rowelementtype = this.options.rowelementtype || 'tr';
        this.cellelementtype = this.options.cellelementtype || 'td';
        this.headerelement = this.createHeader();
        this.lastselected = null;
        this.lastactive = -1;
        this.dragger = new JKDrag({
            ondrag: function(dragger, event, element, position) {
                        var ymove = position.y;
                        var elsize = $(dragger.ghostelement).getSize();
                        var maindivpos = $(element.offsetParent).getPosition();
                        if (position.y < 0 || (position.y + elsize.size.y) > maindivpos.scrollHeight ) {
                            ymove = false;
                        }
                        return { x: false, y: ymove }
                    },
            ondrop: function(dragger, event, playlist) {
                        if (dragger.currentlyover != null) {
                            var pos = dragger.draggingelement.getProperty('playlistposition');
                            var newposition = dragger.currentlyover.getProperty('playlistposition')-1;
                            var i = 0; //playlist.playlistLength()-1; 
                            var moves = new Array();
                            var stop = playlist.playlistLength();
                            var positionoffset = 0;
                            var toggled = false;
                            
                            /* The logic required to move tracks around the playlist is very complicated
                             * if you are moving a track from above the insert point - you need to 
                             * subtract one from your next moved track index - as all tracks below the current
                             * track but above the insert point will shift up.  When you cross the insert
                             * point - you need to clear your track index offset, and increase your insert
                             * point by one, so the next track is inserted in the correct location.  Then
                             * for each remaining track you move you need to increase the insert point index
                             * by one, so you add the tracks in the correct order.  My head hurts.
                             */
                            while(i < stop) {

                                if (playlist.items[i].selected == true) {
                                    if (newposition < 0) {
                                        newposition = 0;
                                    }
                                    moves.push({ start: (i + positionoffset), end: newposition});
                                    if (i < newposition) {
                                        positionoffset--;
                                    }
                                    if (i > newposition ) {
                                        newposition++;
                                    }
                                }
                                if (i == newposition && toggled == false) {
                                    positionoffset = 0;
                                    newposition++;
                                    toggled=true;
                                }
                                i++;
                            }
                            for (var j = 0; j < moves.length; j++) {
                                playlist.davemphandle.moveTrack(moves[j].start, moves[j].end);
                            }
                        }
                    },
            ghostclass: 'ghost',
            draggingclass: 'dragging',
            hoverclass: 'draghover',
            hideoriginal: false
        });
        var func = function (event) {
            return this.keypressHandler(event);
        };
        //this.enclosingelement.addEvent('KeyUp', func.bindAsEventListener(this));
        document.addEvent('keyup', func.bindAsEventListener(this));
        // this just kill's the 'go back' funcitonality normally triggered by the backspace key
        // who thought that was a good idea?  
        document.onkeypress= function(e) {
             var pK = e ? e.which : window.event.keyCode;
             return ( pK != 8 );
        };
        
    },
    
    setPlaylistLength: function(len) {
        if(this.items.length > len) {
            for (var i = this.items.length - 1; i >= len ; i-- ) {
                this.deleteTrack(i);
            }
        }
    },
    playlistLength: function() {
        return this.items.length;
    },
    
    createHeader: function() {
        var row = new Element(this.rowelementtype);
        for ( var i=0; i < this.metadata.length ; i++ ) {
            var cell = new Element(this.cellelementtype);
            row.addClass('playlist_header');
            cell.addClass(this.metadata[i]);
            var text = this.metadata[i].substring(0,1).toUpperCase() + this.metadata[i].substring(1);
            cell.appendText(text);
            row.appendChild(cell);
        }
        if (this.headerelement != null) {
            this.headerelement.remove();
        } 
        this.headerelement = row;
        if (this.enclosingelement.firstChild != null) {
            this.headerelement.injectBefore(this.enclosingelement.firstChild);
        } else {
            this.enclosingelement.appendChild(this.headerelement);
        }
    },
    
    createRow: function(dataobject, element) {
        var row = element;
        if (row == null) {
            row = new Element(this.rowelementtype);
        }
        row.disableSelection();
        row.addClass('playlist_row')
        var child = null;
        while ( child = row.firstChild ) {
            child.remove();
        }
        for ( var i=0; i < this.metadata.length ; i++ ) {
            var cell = new Element(this.cellelementtype);
            cell.addClass('playlist_field');
            cell.addClass(this.metadata[i]);
            if (dataobject[this.metadata[i]] != null) {
                cell.appendText(dataobject[this.metadata[i]]);
            }
            row.appendChild(cell);
        }
        return(row);
    }, 
    
    insertTrack: function(position, dataobject) {
        var datarow = this.createRow(dataobject);
        var itemobject = new Object;
        itemobject.data = dataobject;
        itemobject.element = datarow;
        itemobject.selected = false;
        itemobject.element.setProperty('playlistposition', position);
        if (position >= this.items.length) {
            // we are just appending.
            //this.items.push(dataobject);
            //this.itemelements.push(datarow);
            // since we are appending - and position could actually be greater than the length, we set position
            // so our returned position is correct
            this.items.push(itemobject);
            position = this.items.length - 1;
            this.enclosingelement.appendChild(datarow);
        } else {
            datarow.injectBefore(this.items[position].element);
            this.items.splice(position, 0, itemobject);
            for (var i = 0; i < this.items.length; i++) {
                this.items[i].element.setProperty('playlistposition', i);
            }
        }
        var func = function(event) {  this.selectRowHandler(event, itemobject) };
        var dragfunc = function(event) {
                            this.selectRowHandler(event, itemobject); 
                            this.dragger.startDrag(itemobject.element, event, this) 
                        };
        //this.items[position].element.addEvent('click', func.bindAsEventListener(this));
        this.items[position].element.addEvent('mousedown', dragfunc.bindAsEventListener(this));
        this.dragger.addDragover(this.items[position].element);
        
        return position;
    },
    
    setTrackInfo: function(position, dataobject) {
        if (position < this.items.length) {
            this.items[position].data = dataobject;
            var datarow = this.createRow(dataobject, this.items[position].element);
            this.items[position].element.replaceWith(datarow);
        } else {
            this.insertTrack(position, dataobject);
        }
    },
    
    deleteTrack: function(position) {
        if (position < this.items.length) {
            var itemobject = this.items[position];
            this.items.splice(position, 1);
            this.enclosingelement.removeChild(itemobject.element);
        }
    },
    
    moveTrack: function(oldpos, newpos) {
        if (this.items[oldpos] != null) {
            var data = this.items[oldpos].data;
            this.deleteTrack(oldpos);
            this.insertTrack(newpos, data);
        }
    },
    
    setActive: function(pos) {
        if ( pos != this.lastactive && pos > -1 && (this.items.length > 0)) {
            if (this.lastactive >= 0 && this.lastactive < this.items.length) {
                this.items[this.lastactive].element.removeClass('active_track');
            }
            if (pos < this.items.length) {
                var top = this.items[pos].element.offsetTop - 120; 
                if (top < 0) top = 0;
                var scrollarea = $(this.enclosingelement.parentNode);
                var bottom = scrollarea.scrollTop + scrollarea.getSize().size.y;
                if (scrollarea.scrollTop > top || bottom < (top + 120)) {
                    var scroller = new Fx.Scroll(scrollarea);
                    scroller.scrollTo(0, top);
                }
                this.items[pos].element.addClass('active_track');
                this.lastactive = pos;
            }
        }
    },
    
    acceptItems: function (items) { 
        var position = this.items.length;
        for (var i=0; i < items.length ; i++) {
            if (items[i]['id'] != null) {
                this.davemphandle.addTrackToPlaylist(items[i]['library'] + ":" + items[i]['id'], position++);
            }
        }
    },
    
    setSelection: function (item, selected) {
        item.selected = selected;
        if (selected) {
            item.element.addClass('selected');
        } else {
            item.element.removeClass('selected');
        }
    },
    
    selectRowHandler: function (event, item) {
        //alert(position + " click with meta: " + event.metaKey);
        var position = this.items.indexOf(item);
        if (this.items[position]) {
            if (event.shiftKey == true && this.lastselected != null) {
                var min = (position < this.lastselected) ? position : this.lastselected;
                var max = (position > this.lastselected) ? position : this.lastselected;
                //alert(min + " -> " + max);
                for (var i = min; i <= max; i++) {
                    this.setSelection(this.items[i],true);
                }
            } else if (event.metaKey == true) {
                if (this.items[position].selected) {
                    this.setSelection(this.items[position],false);                   
                } else {
                    this.setSelection(this.items[position],true);                   
                }
            } else {
                var now = new Date().getTime();
                if (this.lastselected == position && now < (this.lastclick+500)) {
                    //alert(now + " - " + this.lastclick);
                    
                    this.davemphandle.play(position);
                    //alert(position);
                }
                if (this.items[position].selected == false) {
                    // no meta key
                    for (var i = 0; i < this.items.length ; i++) {
                        this.setSelection(this.items[i],false);                   
                    }
                    this.setSelection(this.items[position],true);
                }
            }
            if (this.items[position].selected) {
                this.lastselected = position;
                this.lastclick = new Date().getTime();
            }
        }
    },
    
    keypressHandler: function (event) {
        var ev = new Event(event);
        //alert(this.lastselected);
        if (ev.code == 8) {
            var i = this.items.length-1;
            while (i >= 0) {
                if (this.items[i].selected) {
                    this.davemphandle.removeTrackFromPlaylist(i);
                }
                i--;
            }
            //event.stop();
            return(false);
        } else if (ev.code == 65) {
            
            if (ev.control) {
                for (var i = 0 ; i < this.items.length ; i++) {
                    if (ev.shift) {
                        this.setSelection(this.items[i], false);
                    } else { 
                        this.setSelection(this.items[i], true);
                    }
                }
                return(false);
            } 
        }
        //alert("Received a keyup!" + ev.code);
    }
});

function debug(message) {
    var debug = $('debug');
    if (debug.getProperty('active') == 'true') {
        $('debug').appendText(message);
        $('debug').appendChild(new Element('br'));
        $('debug').scrollTop = $('debug').scrollHeight;
    }
}