/*
    mootools based Tree structure. 
    Written by Jay Kuri
    Copyright (c) 2006 Jay Kuri

    This work is licensed under the Creative Commons Attribution 2.5 License.
    To view a copy of this license, visit
    http://creativecommons.org/licenses/by/2.5/ or send a letter to Creative
    Commons, 543 Howard Street, 5th Floor, San Francisco, California, 94105,
    USA.

*/

var TreeNode = new Class({
    
    initialize: function(nodeelement, options) {
        
        this.nodeelement = $(nodeelement);
        this.parentnode = null;
        this.options = new Object;
        this.stayexpandable = false;
        this.expandable = false;
        this.childnodes = new Array;
        this.selected = false;
        this.expanded = false;
        this.data = null;
        this.childblock = null;
        this.expansionindicator = new Element('div').setHTML("&nbsp;");
        var obj = this;
        this.toggleclickfunction = function(e) { obj.toggle(); return false; }
        
        if ($type(this.nodeelement.firstChild) == 'element') {
            this.labelelement = $(this.nodeelement.firstChild);
        } else {
            this.labelelement = $(this.nodeelement);
        }
        this.expansionindicator.setStyle('display', 'inline');
        this.expansionindicator.addClass('tree_indicator');
        this.expansionindicator.addClass('tree_indicator_leaf');
        this.nodeelement.addClass('tree_branchnode');
        this.nodeelement.removeClass('tree_leafnode');
        this.nodeelement.insertBefore(this.expansionindicator, this.nodeelement.firstChild);
        this.nodeelement.addClass('tree_leafnode');
        
        if (options != null) {
            for (var option in options) {
                this.options[option] = options[option];
            }
        }
        if (this.options.nodeclass == null) {
            this.options.nodeclass = 'tree_node';
        }
    },
    
    getNodeElement: function() {
        return this.nodeelement;
    },

    setData: function(value) {
        this.data = value;
        return this.data;
    },
    
    getData: function() {
        return this.data;
    },
    
    setOption: function(name, value) {
        this.options[name] = value;
    },
    
    getOption: function(name) {
        return this.options[name];
    },
    
    getAllOptions: function() {
        return this.options;
    },
    
    deleteOption: function(name) {
        delete this.options[name];
    },
    
    showLabel: function() {
        this.labelelement.setStyle('display', 'block');
        this.expansionindicator.setStyle('display', 'block');
    },
    
    hideLabel: function() {
        this.labelelement.setStyle('display', 'none');
        this.expansionindicator.setStyle('display', 'none')
    },
    
    setExpandable: function(expandable, stayexpandable) {
        this.stayexpandable = stayexpandable || false;
        this.expandable = expandable;
        if (this.expandable) {
            this.expansionindicator.addClass('tree_indicator_closed');
            this.expansionindicator.removeClass('tree_indicator_leaf');
            if (this.childblock == null) {
                this.addChildBlock();
            }
            var obj = this;
            $(this.expansionindicator).addEvent('click', this.toggleclickfunction );
        } else {
            this.expansionindicator.removeClass('tree_indicator_closed');
            this.expansionindicator.addClass('tree_indicator_leaf');   
            $(this.expansionindicator).removeEvent('click', this.toggleclickfunction );
            
        }
    },
    
    addChildBlock: function(element) {
        if (element == null) {
            this.childblock = new Element('div');
        } else {
            this.childblock = $(element);
        }

        this.childblock.addClass(this.options.nodeclass);
        this.nodeelement.appendChild(this.childblock);
        this.childblock.setStyle('display','none');
    },

    addChildElement: function(element, position) {
        var node = new TreeNode(element, this.options);
        return this.addChild(node, position);
    },

    addChild: function(treenode, position) {
        if (this.childblock == null) {
            this.addChildBlock();
        }
        if (position == null) {
            position = this.childnodes.length;
        }
        if (position >= this.childnodes.length) {
            this.childnodes.push(treenode);
            this.childblock.appendChild(treenode.nodeelement);
        } else {   
            this.childnodes.splice(position,0,treenode);
            this.childblock.insertBefore(treenode.nodeelement, this.childnodes[position].nodeelement);
        }
        if (!this.expandable) {
            this.setExpandable(true);
        }
        treenode.parentnode = this;
        if (this.isSelected()) {
            treenode.select();
        } else {
            treenode.deselect();
        }
        return treenode;
    },
    
    removeChild: function(treenode) {
        var foundat = -1;
        //alert(treenode.getOption('info'));
        for(i=0; i < this.childnodes.length; i++)
        {
            if (this.childnodes[i] == treenode) {
                //alert("foundat: " + i);
                foundat = i;
            }
        } 
        if ( foundat != -1) {
            this.removeChildAt(foundat);
        }
        return treenode;
    },
    
    removeChildAt: function(position) {
        //alert(position);
        if (this.childnodes.length > position) {
            if (this.childblock != null ) {
                this.childblock.removeChild(this.childnodes[position].nodeelement);
            }
            this.childnodes[position].parentnode = null;
            this.childnodes.splice(position, 1);
            if (this.childnodes.length == 0) {
                if (this.stayexpandable == false ) {
                    this.setExpandable(false);
                } else {
                    this.collapse();
                }
            }
        }
    },
    
    reparent: function(treenode, position) {
        this.parentnode.removeChild(this);
        treenode.addChild(this, position);
    },
    
    isSelected: function() {
        return(this.selected);
    },
    
    select: function(selectchildren) {
        //alert("calling select");
        if (this.selected == false) {
            if (this.options.onselect && typeof(this.options.onselect) == 'function') {
                if (this.options.onselect(this) == false) {
                    return;
                }
            }
            this.selected = true;
            this.labelelement.addClass('tree_node_selected');
        }
        if (selectchildren == true) { 
            this.childnodes.each(function(node) { node.select(true); });
        }
    },
    
    deselect: function(deselectchildren) {
        if (this.selected == true) {
            if (this.options.ondeselect && typeof(this.options.ondeselect) == 'function') {
                if (this.options.ondeselect(this) == false) {
                    return;
                }
            }
            this.selected = false;
            this.labelelement.removeClass('tree_node_selected');
            if (this.parentnode.isSelected()) {
                this.parentnode.deselect();
            }
        }
        if (deselectchildren == true) { 
            this.childnodes.each(function(node) { node.deselect(true); });
        }
        
    },
    
    getChildren: function() {
        return(this.childnodes);
    },
    
    getSelectedChildren: function() {
        var selectedchildren = new Array();
        if (this.childnodes.length > 0 ) {
            for (var i = 0; i < this.childnodes.length; i++) {
                if (this.childnodes[i].isSelected()) {
                    selectedchildren.push(this.childnodes[i]);
                }
                selectedchildren.extend(this.childnodes[i].getSelectedChildren());
            }
        }
        return(selectedchildren);
    },
    
    expand: function() {
        if (!this.expanded) {
            if (this.expandable == true && this.options.onexpand && typeof(this.options.onexpand) == 'function') {
                if (this.options.onexpand(this) == false) {
                    return;   
                }
            }
            //alert("display is " + display);
            if (this.expandable && this.childblock != null) {
                this.expanded = true;
                this.expansionindicator.removeClass('tree_indicator_closed');
                this.expansionindicator.addClass('tree_indicator_open');
                this.childblock.setStyle('display','block');
            }
        }
    },
      
    collapse: function() {
        if (this.expanded) {
            if (this.options.oncollapse && typeof(this.options.oncollapse) == 'function') {
                if (this.options.oncollapse(this) == false) {
                    return;
                }
            }
            if (this.childblock != null) {
                this.expanded = false;
                this.expansionindicator.removeClass('tree_indicator_open');
                this.expansionindicator.addClass('tree_indicator_closed');
                this.childblock.setStyle('display','none');
            }
        }
    },
    
    expandAllChildren: function() {
        if (this.childblock != null) {
            for(var i=0; i < this.childnodes.length; i++)
            {
                this.childnodes[i].expandAllChildren();
            }
            this.expand();
        }
    },
    
    collapseAllChildren: function() {
        if (this.childblock != null) {
            for(i=0; i < this.childnodes.length; i++)
            {
                this.childnodes[i].collapseAllChildren();
            }
            this.collapse();
        }
    },
    
    toggle: function() {
        if ((this.childblock != null && this.childblock.getStyle('display') == 'none' ) ||
            (this.expandable == true && this.childblock == null))
        {
            this.expand();
        } else {
            this.collapse();
        }
    }

});
